
/*****************************************************************
PREMIA
5 
******************************************************************/

/* to be put first, because optype.h redefines OUT IN
 already defined in windows.h */
#if defined(_WIN32) && !defined(_CYGWIN)
#include <windows.h>
#undef IN
#undef OUT
#endif

#include "optype.h"
#include "var.h"

#include "method.h"
#include "test.h"
#include "timeinfo.h"

#include "tools.h"
#include "ftools.h"

#include "random.h"

#ifdef _MSC_VER
#include "../../Windows/winconfig.h"
#else
#include "../../Unix/config.h"
#endif


extern Model BS1D_model;
extern Model BS2D_model;
extern Model DUP1D_model;
extern Model MER1D_model;
extern Model HW1D_model;
extern Model HES1D_model;
Model  *models[]=
{
  &BS1D_model,
  &BS2D_model,
  &DUP1D_model,
  &MER1D_model,
  &HW1D_model,
  &HES1D_model,
  NULL
};

extern Family STD_family;
extern Family LIM_family;
extern Family LIMDISC_family;
extern Family DOUBLIM_family;
extern Family PAD_family;
extern Family STD2D_family;
Family *families[]=
{
  &STD_family,
  &LIM_family,
  &LIMDISC_family,
  &DOUBLIM_family,
  &PAD_family,
  &STD2D_family,
  NULL
};

extern  Pricing BS1D_STD_pricing;
extern  Pricing BS1D_LIM_pricing;
extern  Pricing BS1D_LIMDISC_pricing;
extern  Pricing BS1D_PAD_pricing;
extern  Pricing BS1D_DOUBLIM_pricing;
extern  Pricing BS2D_STD2D_pricing;
extern  Pricing DUP1D_STD_pricing;
extern  Pricing MER1D_STD_pricing;
extern  Pricing HW1D_STD_pricing;
extern  Pricing HES1D_STD_pricing;

Pricing *pricings[]=
{
  &BS1D_STD_pricing,
  &BS1D_LIM_pricing,
  &BS1D_LIMDISC_pricing,
  &BS1D_PAD_pricing,
  &BS1D_DOUBLIM_pricing,
  &BS2D_STD2D_pricing,
  &DUP1D_STD_pricing,
  &MER1D_STD_pricing,
  &HW1D_STD_pricing,
  &HES1D_STD_pricing,
  NULL
};

extern TimeInfo computation_time_info;

FILE *out_stream;
char premiasrcdir[256];
char premiadir[256];
char premiapersodir[256];
#ifdef _WIN32
const char *path_sep = "\\";
#endif
#ifndef _WIN32
const char *path_sep = "/";
#endif

char premiamandir[256];

char PREMIA_OUT[256]="";
char GNUPLOT_DAT[256]="";
char TITLES_TEX[256]="";
char GNUPLOT_SCREEN_PLT[256]="";
char GNUPLOT_FILE_PLT[256]="";
char GNU_TEX[256]="";
char PREMIA_LOG[256]="";
char SESSION_LOG[256]="";


int main(int argc, char *argv[])
{
	
  Planning          plan;
  Model*            pt_model;
  Option*           pt_option;
  Pricing*          pt_pricing;
  PricingMethod*    pt_method;
  DynamicTest*      pt_test;
  PricingMethod*    pt_methods_available[MAX_METHODS];
  char             *tmp,*tmp1;
  int user;
  char             FileRead[MAX_LINE][MAX_CHAR_LINE];
	

#if defined(_WIN32) && !defined(_CYGWIN)

  LPSTR tail;
  
  GetModuleFileName(GetModuleHandle(NULL),premiapersodir, 256);
  
  if ((tail = strrchr (premiapersodir, '\\')) != (LPSTR) NULL)
  {
	*tail = '\0';
  }
  
  strcpy(premiadir, premiapersodir);

  if ((tail = strrchr (premiadir, '\\')) != (LPSTR) NULL)
  {
	*tail = '\0';
  }
  sprintf(premiamandir, "%s\\Src", premiadir );
  sprintf(premiasrcdir, "%s\\Src", premiadir);
#else /* end _WIN32 */
  
  sprintf(premiamandir, "%s", PREMIA_MAN_PDF_DIR);
  sprintf(premiasrcdir, "%s", PREMIA_SRC_DIR);
  sprintf(premiapersodir, "%s", PREMIA_HOME_DIR);
  sprintf(premiadir, "%s", PREMIADIR);

#endif /*_WIN32 */
  
  sprintf(PREMIA_OUT,"premia.out");
  sprintf(GNUPLOT_DAT,"%s%sgnupremia.dat",premiapersodir,path_sep);
  sprintf(TITLES_TEX,"titles.tex");
  sprintf(GNUPLOT_SCREEN_PLT,"%s%sgnutoscreen.plt",premiapersodir,path_sep);
  sprintf(GNUPLOT_FILE_PLT,"%s%sgnutofile.plt",premiapersodir,path_sep);
  sprintf(GNU_TEX,"%s%sgnupremia.tex",premiapersodir,path_sep);
  sprintf(PREMIA_LOG,"%s%spremia.log",premiapersodir,path_sep);
  sprintf(SESSION_LOG,"%s%ssession.log",premiapersodir,path_sep);
	
  if( argc==2 ){
		

	(void)ReadInputFile(argv[1],FileRead);
	(void)InputMode(&user);
	(void)WellcomeMsg(user);
	if ((InitErrorMsg()==OK)&&(InitVar()==OK)&&(InitMc()==OK))
	{

	  (void)ExitVar();
	  (void)FreeErrorMsg();
	}

		


  }	else {
			

	(void)InputMode(&user);
	(void)WellcomeMsg(user);
	if ((InitErrorMsg()==OK)&&(InitVar()==OK)&&(InitMc()==OK))
	{
	  do
	  {
		(void)ResetPlanning(&plan);

		plan.Action=ChooseAction();
									
		if (OutputFile(&out_stream)==OK)
		{
		  if (SelectModel(user,&plan,models,&pt_model)==OK)
		  {
			if (SelectOption(user,&plan,families,pt_model,pricings,&pt_option)==OK)
			{
			  if (SelectPricing(user,pt_model,pt_option,pricings,&pt_pricing)==OK)
			  {
				while(1)
				{
				  if (SelectMethod(user,&plan,pt_pricing,pt_option,pt_model,&pt_method)==OK)
				  { 
							
					if (SelectTest(user,&plan,pt_pricing,pt_option,pt_model,pt_method,&pt_test)==OK)	{	
				      						 
					  if (GetTimeInfo(user,&plan,&computation_time_info)==OK)
					  {
						/*action=ChooseAction();*/
						if ((plan.Action=='p')||(( plan.Action=='t')&&(GetTest(user,&plan,pt_pricing,pt_option,pt_test)==OK))   )
						{
						  (void)ShowPlanning(NAMEONLYTOFILE,&plan);
						  (void)Action(plan.Action,pt_model,pt_option,pt_pricing,pt_method,pt_test,NAMEONLYTOFILE,&plan,&computation_time_info);
									
						  Fprintf(TOSCREEN,"\nComputing...\n");
						  Iterate(&plan,&(plan.Par[0]),0,plan.Action,pt_model,pt_option,pt_pricing,pt_method,pt_test,TOFILE,&computation_time_info);
						  pt_methods_available[plan.NumberOfMethods]=pt_method;
						  if (plan.Action=='t' || MoreAction(&(plan.NumberOfMethods))==WRONG)break;
						}
					  }
					       
					}
					else break;
				  }else break;
				}
			  }
			}
		  }
		}
			
		fclose(out_stream);
	  
		if ((plan.Action=='p') && (plan.VarNumber>0))
		  (void)BuildGnuStuff(plan,pt_model,pt_option,pt_pricing,pt_methods_available); 
					      
	  
		if (plan.Action=='t')
		{
							
		  (void)FreeTest(pt_test); 
		  (void)BuildGnuStuffTest(pt_model,pt_option,pt_pricing,pt_method,pt_test); 
		}
	  }while (NextSession(&plan,plan.Action,user)==OK);
	}
	
	(void)ExitVar();
	(void)FreeErrorMsg();
  }
	
	
  return OK;
}




