#!/bin/bash
#______________________________________________________________
# PURPOSE: Archive output of PREMIA. It produces two files:
#          (i) a .pdf file of the last run
#          (ii) a summary of all archived tests for RoutineName,
#                 named "RoutineName_testl.pdf".
#
# USAGE:   Do either (1) chmod +x bash_archive; bash_archive
#                or (2) bash bash_archive
# VERSION: 1.1 (28/1/99) VK
#          1.2 (14/2/99) VK
#____      1.3 (25/5/99) CM__________________________________________________________

#!!!!USER DEPENDENT!!!!
#begin
gnupath=C:/Gnuplot/
ps2pdfpath=c:/gstools/gs5.50/
#end

echo "Enter your user name (lowercase):"
read USER

ppath="$(grep %% gnupremia.tex | gawk -F% '{print $3}' | gawk -F/ 'NF>2')"
Pricing="$(grep %% gnupremia.tex | gawk -F% '{print $3}' | gawk -F/ 'NF==2')"
PricingOrTest="`grep % gnupremia.tex | gawk -F% 'NF==2' | gawk -F% '{print $2}' | wc -l`"

if [$PricingOrTest -eq 1 ] ; then
      echo "Archiving the test......."
      RoutineName="$(grep %%% gnupremia.tex | gawk -F% '{print $4}' | gawk -F/ 'NF==1' | gawk -F_ 'NF==3')"
#echo "routine name : ${RoutineName} fin routine name"
      let "nomethods = 0"
else
    nomethods="`grep %% gnupremia.tex | wc -l`"
    let "nomethods=nomethods -2"
    echo "The number of routines compared is: ${nomethods}"
    if [ $nomethods -eq 1 ] ; then
    RoutineName="$(grep %% gnupremia.tex | gawk -F% '{print $3}' | gawk -F/ 'NF==1')"
    else
    RoutineName="$(grep %% gnupremia.tex | gawk -F% '{print $3}' | gawk -F/ 'NF==2' | gawk -F/ '{print $1}')"
    fi
fi

echo "routine name : ${RoutineName} fin routine name"

filename="gnupremia.tex"

if [ -e $filename ]; then
  ${gnupath}gnuplot gnutofile.plt
  latex \\scrollmode\\input gnupremia.tex
  latex \\scrollmode\\input gnupremia.tex
  dvipdfm gnupremia.dvi
else
  echo "ERROR: ${filename} does not exist"
  exit
fi

#__________________ ARCHIVE LAST RUN  __________________________
ls ${ppath} > list_all_files

# find the number of existing "_trial#.pdf" files

nofiles="`grep -c ${RoutineName}_trial_${USER} list_all_files`"

if [$nomethods -eq 1]; then
    echo "The number of existing trials for routines ${RoutineName} is: $nofiles"
else
    echo "The number of existing comparative tests  ${RoutineName} is: $nofiles"
fi

let nofiles=nofiles+1

ext_trial_no="_trial_${USER}_${nofiles}.pdf"

while [ -e ${ppath}${RoutineName}${ext_trial_no} ] ; do
  echo "File ${RoutineName}${ext_trial_no} already exists in ${ppath}."
  echo "  "
  echo "    Re-adjusting numbering ..."
  let nofiles=nofiles - 1
  unset -v ext_trial_no
  ext_trial_no="_trial_${USER}_${nofiles}.pdf"
done

if [ ! -e ${ppath}${RoutineName}${ext_trial_no} ]; then
  mv gnupremia.pdf ${ppath}${RoutineName}${ext_trial_no}
fi

echo "The source file is:  `find ${ppath} -name ${RoutineName}.c -print`"


#________________________ MAKE  SUMMARY ___________________________#
# Add this pdf file to the summary file: RoutineName_testl.tex

no_trial_files="`find ${ppath} -name ${RoutineName}_trial\*.pdf -print | wc -l`"
echo "The number of all trials archived so far is ${no_trial_files}"

echo "\\documentclass[12pt,a4paper]{article}" > ${RoutineName}_testl.tex
#echo "\\usepackage{alltt}" >> ${RoutineName}_testl.tex
#echo "\\usepackage[dvipdfm]{color}" >>  ${RoutineName}_testl.tex
#echo "\\usepackage[colorlinks,filecolor=blue,dvipdfm]{hyperref}" >> ${RoutineName}_testl.tex
echo "\\input premiamble" >> ${RoutineName}_testl.tex
echo "\\begin{document}" >> ${RoutineName}_testl.tex
echo "\\begin{center}" >> ${RoutineName}_testl.tex
echo "{\\bf\\LARGE Pricing Method Trials}"  >> ${RoutineName}_testl.tex
echo "\\begin{verbatim}                  ${Pricing}${RoutineName}.c \\end{verbatim} "  >> ${RoutineName}_testl.tex
echo "\\end{center}" >> ${RoutineName}_testl.tex
echo "\\begin{alltt}" >> ${RoutineName}_testl.tex

dummy=1

for i in `find ${ppath} -name ${RoutineName}_trial\*.pdf  -print |  gawk -F/ '{print $6}' `
do
  echo " ${RoutineName}.c: \\href{${i}}{Trial ${dummy} } " >> ${RoutineName}_testl.tex
  let dummy=dummy+1
done

echo "\\vspace{1cm}" >> ${RoutineName}_testl.tex
echo "\\href{${RoutineName}_src.pdf}{Source} " >> ${RoutineName}_testl.tex
echo "\\href{${RoutineName}_doc.pdf}{Back to ${RoutineName}_doc.pdf } \\\\ " >> ${RoutineName}_testl.tex
echo "\\end{alltt}" >> ${RoutineName}_testl.tex
echo "\\input premiaendnobib" >> ${RoutineName}_testl.tex
echo "\\end{document}" >> ${RoutineName}_testl.tex

latex \\scrollmode\\input ${RoutineName}_testl.tex
latex \\scrollmode\\input ${RoutineName}_testl.tex

dvipdfm ${RoutineName}_testl.dvi
mv ${RoutineName}_testl.pdf $ppath

#________________________ CLEAN UP _____________________________
# Clean up uncesessary files:
rm list_all_files
rm *.tmp  *.ps *.log  *.dvi  *.aux *.out
rm  *.tex



echo "  "
echo "  "
echo "The last archived file is  ${ppath}${pname}${ext_trial_no} "
echo "  "
echo "The archived summary file is: ${ppath}${RoutineName}_test.pdf"
echo "  "
echo "  "
echo "To view a copy of this run execute:  acroread ${RoutineName}${ext_trial_no}"
